require "behaviours/follow"
require "behaviours/wander"
require "behaviours/faceentity"
require "behaviours/chaseandattack"


local AgethaBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

local MIN_FOLLOW = 2
local MAX_FOLLOW = 5
local MED_FOLLOW = 4
local MAX_WANDER_DIST = 10
local MAX_CHASE_TIME = 10

function AgethaBrain:OnAttacked(attacker)
    self.inst.components.combat:SetTarget(attacker)
end

function AgethaBrain:OnAttackOther(target)
    self.inst.components.combat:SetTarget(target)
end


local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

function AgethaBrain:OnStart()

    local root = PriorityNode(
    {
		ChaseAndAttack(self.inst, MAX_CHASE_TIME),
		Follow(self.inst, function() return self.inst.components.follower.leader end, MIN_FOLLOW, MED_FOLLOW, MAX_FOLLOW, true),
		FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn),
        Wander(self.inst, function() return Point(GetPlayer().Transform:GetWorldPosition()) end , MAX_WANDER_DIST)        
    }, .5)
        
    self.bt = BT(self.inst, root)
	
    self.inst:ListenForEvent("attacked", function(inst, data) self:OnAttacked(data.attacker) end)  
    self.inst:ListenForEvent("onattackother", function(inst, data) self:OnAttackOther(data.target) end)
         
end


return AgethaBrain